//=========================================================================
//
//	FILE:				CP_Exceptions.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Various macros, routines and other defines to help
//						manage exceptions and exception handling in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 226 $
//						Latest Checkout:  $Date: 2005-07-02 15:14:01 -0700 (Sat, 02 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Exceptions
#define _H_CP_Exceptions
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Debugging.h"
#include "CP_DebugStr_Defines.h"
#include "CP_ErrorManager.h"
#include "CP_OSStatus.h"
#include "CP_Strings.h"
#include "CP_Types.h"

	// Standard library
#include <exception>
#include <map>
#include <stdexcept>
#include <string>

CPLAT_Begin_Namespace_CPLAT

//class CP_String;
	
#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_Exceptions ===
#pragma mark -
#endif

class CP_EXPORT CP_MemoryException : public std::bad_alloc
{ 
public: 
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MemoryException(
												const char *inFunction,
												int inLine,
												const char *inFile )
													: fFunction(inFunction), fFile(inFile), fLine(inLine) { fMessageBuffer[0] = '\0'; }
	//@}

	//! @name Getters
	//@{
	virtual const char					*what() const throw();
	//@}

	// class data
public:
	const char 							*fFunction;
	const char 							*fFile;
	int									fLine;
	char								fMessageBuffer[ 512 ];
};


class CP_EXPORT CP_OSErrorException : public std::runtime_error
{ 
public: 
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_OSErrorException(
												OSStatus inStatus,
												const char *inFunction,
												int inLine,
												const char *inFile )
													: std::runtime_error(""), fFunction(inFunction),
														fFile(inFile), fLine(inLine)		{ fStatus = inStatus; fMessageBuffer[0] = '\0'; }
	//@}

	//! @name Getters
	//@{
			OSStatus					GetOSStatus() const								{ return( fStatus ); }
	virtual const char					*what() const throw();
	//@}
			
	// class data
public:
	OSStatus							fStatus;
	const char 							*fFunction;
	const char 							*fFile;
	int									fLine;
	char								fMessageBuffer[ 512 ];
};

class CP_EXPORT CP_CancelException : public std::runtime_error
{ 
public: 
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_CancelException(
												const char *inFunction,
												int inLine,
												const char *inFile )
													: std::runtime_error(""), fFunction(inFunction),
														fFile(inFile), fLine(inLine) 		{ fMessageBuffer[0] = '\0'; }
	//@}

	//! @name Getters
	//@{
	virtual const char					*what() const throw();
	//@}

	// class data
public:
	const char 							*fFunction;
	const char 							*fFile;
	int									fLine;
	char								fMessageBuffer[ 512 ];
};

class CP_EXPORT CP_BoundsException : public std::runtime_error
{ 
public: 
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_BoundsException(
												const char */*inMessage*/,
												const char *inFunction,
												int inLine,
												const char *inFile )
													: std::runtime_error(""), fFunction(inFunction),
														fFile(inFile), fLine(inLine) 		{ fMessageBuffer[0] = '\0'; }
	//@}

	//! @name Getters
	//@{
	virtual const char					*what() const throw();
	//@}

	// class data
public:
	const char 							*fFunction;
	const char 							*fFile;
	int									fLine;
	char								fMessageBuffer[ 512 ];
};

class CP_EXPORT CP_NotFoundException : public std::runtime_error
{ 
public: 
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_NotFoundException(
												const char */*inMessage*/,
												const char *inFunction,
												int inLine,
												const char *inFile )
													: std::runtime_error(""), fFunction(inFunction),
														fFile(inFile), fLine(inLine) 		{ fMessageBuffer[0] = '\0'; }
	//@}

	//! @name Getters
	//@{
	virtual const char					*what() const throw();
	//@}

	// class data
public:
	const char 							*fFunction;
	const char 							*fFile;
	int									fLine;
	char								fMessageBuffer[ 512 ];
};

#if CP_ThrowOnAssert
class CP_EXPORT CP_AssertException : public std::logic_error
{ 
public: 
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AssertException(
												const char *inFunction,
												int inLine,
												const char *inFile )
													: fFunction(inFunction), fFile(inFile), fLine(inLine) 	{ fMessageBuffer[0] = '\0'; }
	//@}

	//! @name Getters
	//@{
	virtual const char					*what() const throw();
	//@}

	// class data
public:
	const char 							*fFunction;
	const char 							*fFile;
	int									fLine;
	char								fMessageBuffer[ 512 ];
};
#endif	// CP_THROW_ON_ASSERT	

//=========================================================================
//	Exception routines	
//=========================================================================
CP_EXPORT void CP_ThrowErr_( OSStatus inStatus, const char *inFunction, int inLine, const char *inFile );		
CP_EXPORT void CP_ThrowIfNULL_( const void *inPtr, const char *inFunction, int inLine, const char *inFile );
CP_EXPORT void CP_ThrowIfResFail_( const void *inPtr, const char *inFunction, int inLine, const char *inFile );
CP_EXPORT void CP_ThrowRuntime_( const CP_String& inMessage, const char *inFunction, int inLine, const char *inFile );		
CP_EXPORT void CP_ThrowOutOfRange_( const CP_String& inMessage, const char *inFunction, int inLine, const char *inFile );		
CP_EXPORT void CP_ThrowOverflow_( const CP_String& inMessage, const char *inFunction, int inLine, const char *inFile );		
CP_EXPORT void CP_ThrowUnderflow_( const CP_String& inMessage, const char *inFunction, int inLine, const char *inFile );		
CP_EXPORT void CP_ThrowInvalidArgument_( const CP_String& inMessage, const char *inFunction, int inLine, const char *inFile );		
CP_EXPORT void CP_ThrowLength_( const CP_String& inMessage, const char *inFunction, int inLine, const char *inFile );		
CP_EXPORT void CP_ThrowBadCast_( const CP_String& inMessage, const char *inFunction, int inLine, const char *inFile );		

CP_EXPORT void CP_ThrowBounds_( const CP_String& inMessage, const char *inFunction, int inLine, const char *inFile );		
CP_EXPORT void CP_ThrowNotFound_( const CP_String& inMessage, const char *inFunction, int inLine, const char *inFile );		

inline void CP_ThrowIfOSErr_( OSStatus inStatus, const char *inFunction, int inLine, const char *inFile )				{ if( inStatus != CP_noErr ) CP_ThrowErr_( inStatus, inFunction, inLine, inFile ); }	

CP_EXPORT void CP_ThrowIfMemFail_( const void *inPtr, const char *inFunction, int inLine, const char *inFile );

#if TARGET_OS_MAC
	inline void CP_ThrowIfResErr_( const char *inFunction, int inLine, const char *inFile )								{ CP_ThrowIfOSErr_( ::ResError(), inFunction, inLine, inFile ); }	
	inline void CP_ThrowIfMemErr_( const char *inFunction, int inLine, const char *inFile )								{ CP_ThrowIfOSErr_( ::MemError(), inFunction, inLine, inFile ); }	
	
	#if CP_UseDepreciated_10_4_APIs
		inline void CP_ThrowIfQDErr_( const char *inFunction, int inLine, const char *inFile )							{ CP_ThrowIfOSErr_( ::QDError(), inFunction, inLine, inFile ); }
	#endif
	
	CP_EXPORT void CP_ThrowIfCFFail_( const void *inPtr, const char *inFunction, int inLine, const char *inFile );
#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32
	inline void CP_ThrowIfErr_( OSStatus inError, const char *inFunction, int inLine, const char *inFile )				{ if( inError != NO_ERROR ) CP_ThrowErr_( inError, inFunction, inLine, inFile ); }
	inline void CP_ThrowIfErr_( const char *inFunction, int inLine, const char *inFile )								{ CP_ThrowIfErr_( (SInt32) ::GetLastError(), inFunction, inLine, inFile ); }

	inline void	CP_ThrowIfInvalidHandle_( HANDLE inHandle, const char *inFunction, int inLine, const char *inFile )		{ if( inHandle == NULL || inHandle == INVALID_HANDLE_VALUE ) { CP_ThrowIfErr_( (SInt32) ::GetLastError(), inFunction, inLine, inFile ); CP_ThrowErr_( ERROR_INVALID_HANDLE, inFunction, inLine, inFile ); } }

	inline void	CP_ThrowIf_( bool inFlag, const char *inFunction, int inLine, const char *inFile )						{ if( inFlag ) { CP_ThrowIfErr_( (SInt32) ::GetLastError(), inFunction, inLine, inFile ); CP_ThrowErr_( (SInt32) E_FAIL, inFunction, inLine, inFile ); } }
	inline void CP_ThrowIfFailed_( HRESULT inResult, const char *inFunction, int inLine, const char *inFile )			{ if( FAILED(inResult) ) CP_ThrowErr_( inResult, inFunction, inLine, inFile ); }
#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX
	inline void CP_ThrowIfErr_( OSStatus inError, const char *inFunction, int inLine, const char *inFile )				{ if( inError != CP_noErr ) CP_ThrowErr_( inError, inFunction, inLine, inFile ); }

	inline void	CP_ThrowIf_( bool inFlag, const char *inFunction, int inLine, const char *inFile )						{ if( inFlag ) { CP_ThrowIfErr_( (SInt32) -1, inFunction, inLine, inFile ); CP_ThrowErr_( (SInt32) -1, inFunction, inLine, inFile ); } }
#endif

//=========================================================================
//	Macros to throw exceptions	
//=========================================================================

	// Common
#define CP_ThrowErr( inStatus )					CP_ThrowErr_( inStatus, __FUNCTION__, __LINE__, __FILE__  )
#define CP_ThrowIfNULL( inStatus )				CP_ThrowIfNULL_( inStatus, __FUNCTION__, __LINE__, __FILE__ )
#define CP_ThrowIfResFail( inStatus )			CP_ThrowIfResFail_( inStatus, __FUNCTION__, __LINE__, __FILE__ )

	// Standard Library
#define CP_ThrowRuntime( inMsg )				CP_ThrowRuntime_( inMsg, __FUNCTION__, __LINE__, __FILE__ )
#define CP_ThrowOutOfRange( inMsg )				CP_ThrowOutOfRange_( inMsg, __FUNCTION__, __LINE__, __FILE__ )
#define CP_ThrowOverflow( inMsg )				CP_ThrowOverflow_( inMsg, __FUNCTION__, __LINE__, __FILE__ )
#define CP_ThrowUnderflow( inMsg )				CP_ThrowUnderflow_( inMsg, __FUNCTION__, __LINE__, __FILE__ )
#define CP_ThrowInvalidArgument( inMsg )		CP_ThrowInvalidArgument_( inMsg, __FUNCTION__, __LINE__, __FILE__ )
#define CP_ThrowLength( inMsg )					CP_ThrowLength_( inMsg, __FUNCTION__, __LINE__, __FILE__ )
#define CP_ThrowIfOSErr( inStatus )				CP_ThrowIfOSErr_( inStatus, __FUNCTION__, __LINE__, __FILE__ )
#define CP_ThrowBadCast( inMsg )				CP_ThrowBadCast_( inMsg, __FUNCTION__, __LINE__, __FILE__ )

	// CPLAT
#define CP_ThrowBounds( inMsg )					CP_ThrowBounds_( inMsg, __FUNCTION__, __LINE__, __FILE__ )
#define CP_ThrowNotFound( inMsg )				CP_ThrowNotFound_( inMsg, __FUNCTION__, __LINE__, __FILE__ )
#define CP_ThrowIfMemFail( inStatus )			CP_ThrowIfMemFail_( inStatus, __FUNCTION__, __LINE__, __FILE__ )

#if TARGET_OS_MAC
		// MacOS only
	#define CP_ThrowIfResErr( inStatus )		CP_ThrowIfResErr_( inStatus, __FUNCTION__, __LINE__, __FILE__ )
	#define CP_ThrowIfMemErr( inStatus )		CP_ThrowIfMemErr_( inStatus, __FUNCTION__, __LINE__, __FILE__ )
	#define CP_ThrowIfCFFail( inStatus )		CP_ThrowIfCFFail_( inStatus, __FUNCTION__, __LINE__, __FILE__ )
	#define CP_ThrowIfQDErr( inStatus )			CP_ThrowIfQDErr_( inStatus, __FUNCTION__, __LINE__, __FILE__ )
#endif

#if TARGET_OS_WIN32
		// Win32 only
	#define CP_ThrowIfMemErr( inHandle )		CP_ThrowIfInvalidHandle_( inHandle, __FUNCTION__, __LINE__, __FILE__ )

	#define CP_ThrowIf( inFlag )				CP_ThrowIf_( inFlag, __FUNCTION__, __LINE__, __FILE__ )
	#define CP_ThrowIfFailed( inHandle )		CP_ThrowIfFailed_( inHandle, __FUNCTION__, __LINE__, __FILE__ )
#endif

#if TARGET_OS_LINUX
	#define CP_ThrowIf( inFlag )				CP_ThrowIf_( inFlag, __FUNCTION__, __LINE__, __FILE__ )
#endif

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === OSStatus exception ===
#pragma mark -
#endif

//=========================================================================
//	OSStatus exception
//=========================================================================

/*!
*	@class	OSStatus exception
*	@brief	Exception class to throw OSStatus exceptions and can have specific handlers for a particular status code.
*	
*		The class of exceptions is designed to all the user to throw and catch specific
*		OSStatus exceptions.
*	
*		This code snipped show how you can register the exceptions you want to provide
*		a specific catch block handler for.
*
*	\code
*	try
*	{
*		CPLAT::RunOnce< CPLAT::RegisterOSStatus< CP_fnfErr > >();
*		
*		OSStatus		err = CP_fnfErr;
*		
*		CPLAT::ThrowOSStatus( err );
*	}
*	catch( CPLAT::ErrorCode< CPLAT::OSStatus, CP_fnfErr > )
*	{
*		// should end up here
*	}
*	catch(...)
*	{
*		// should not end up here
*	}
*	\endcode
*/

#if ! TARGET_CC_MSVC
	// Construction / destruction
template < class ErrorClass >
struct ErrorClassTraits
{
	typedef typename ErrorClass::ErrorStatus		ErrorStatus;
};   
   
template < class ErrorClass, typename ErrorClassTraits<ErrorClass>::ErrorStatus number >
class ErrorCode : public ErrorClass
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										ErrorCode()
											: ErrorClass( number )			{}
	//@}
};


template < class Exception >
void Throw()
{
	throw Exception();
}

   
template < class ErrorClass >
class ErrorCodeThrower
{
private:
	typedef typename ErrorClassTraits< ErrorClass >::ErrorStatus	ErrorStatus;
 	typedef std::map< ErrorStatus, void(*)() >						ErrorCodeMap;
      
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										ErrorCodeThrower()					{}
	//@}
         
	//! @name Regristration
	//@{
	template < ErrorStatus number >
	void								Register()
										{
											fErrorCodes[ number ] = CPLAT::Throw< ErrorCode< ErrorClass, number > >;
										}
 	//@}
        
	//! @name Throwing
	//@{
	void								Throw( ErrorClass error ) const
										{
											typename ErrorCodeMap::const_iterator		found = fErrorCodes.find(  error  );

											if( found != fErrorCodes.end() )
												return( found->second() );
									
											throw( error );
										}
	//@}
	
private:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
											// Don't allow copy and assignment
										ErrorCodeThrower( const ErrorCodeThrower& );
										ErrorCodeThrower& operator=( const ErrorCodeThrower& );
	//@}

	// class data
private:
	ErrorCodeMap						fErrorCodes;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

template < class ErrorClass >
ErrorCodeThrower<ErrorClass>& CP_ErrorCodeThrower()
{
	static ErrorCodeThrower<ErrorClass>		gErrorCodeThrower;
	
	return( gErrorCodeThrower );
}
   
template < class ErrorClass, typename ErrorClassTraits<ErrorClass>::ErrorNumber number >
void CP_RegisterErrorCode()
{
	CP_ErrorCodeThrower<ErrorClass>().template Register<number>();
}
   
template < class ErrorClass >
void CP_ThrowErrorCode( ErrorClass error )
{
	if( CP_Debugging::CP_DebugNotification::GetReportExceptionOnThrow() )
	{
		CP_ReportError(
			CP_Strings::GetString( kCP_ErrorWarnMessages_ResID, kDebug_CouldNotCompleteOperation_index ),
			CP_Strings::GetString( kCP_ErrorWarnMessages_ResID, kDebug_AnOSStatusExceptionOccurred_index ) );
	}
	
	CP_ErrorCodeThrower<ErrorClass>().Throw( error );
}

template < SInt32/*::OSStatus*/ error >
void CP_RegisterOSStatus()
{
	CP_RegisterErrorCode<OSStatus, error>();
}
   
	// user callable function to throw an OSStatus exception
void CP_ThrowOSStatus( OSStatus );

#endif	// TARGET_CC_MSVC

class CP_StSuppressExceptionThrowNotifications
{
	bool	fSavedReportFlag;
public:
	CP_StSuppressExceptionThrowNotifications() : fSavedReportFlag(CP_Debugging::CP_DebugNotification::GetReportExceptionOnThrow()) 
	{
		CP_Debugging::CP_DebugNotification::SetReportExceptionOnThrow(false);
	}
	~CP_StSuppressExceptionThrowNotifications()
	{
		CP_Debugging::CP_DebugNotification::SetReportExceptionOnThrow(fSavedReportFlag);
	}
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Exceptions
