//=========================================================================
//
//	FILE:				CP_MacOSPtr.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to provide a wrapper around a MacOS Ptr.
//						This class can also be used if QuickTime for Windows
//						is defined.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 225 $
//						Latest Checkout:  $Date: 2005-07-01 17:21:21 -0700 (Fri, 01 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_MacOSPtr
#define _H_CP_MacOSPtr
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_ReferenceCounted.h"
#include "CP_Types.h"

#if TARGET_OS_MAC || CP_Optional_QuickTime_Windows
	#if TARGET_API_MAC_CARBON && __CF_USE_FRAMEWORK_INCLUDES__
		#include <Carbon/Carbon.h>
	#else
		#include <MacMemory.h>
	#endif
#endif

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC || CP_Optional_QuickTime_Windows

/*!
*	@class		CP_MacOSPtr
*	@brief		Memory allocation class based on pointers.
*	
*				A reference counted object to manage blocks of memory using pointers.
*/
class CP_EXPORT CP_MacOSPtr : public CP_M_RefCounted
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MacOSPtr();
										CP_MacOSPtr( UInt32 inSize );
										CP_MacOSPtr( CP_MacOSPtr& inPtrBlock );
										CP_MacOSPtr(
												Ptr inMacPtr, 
												bool inOwnsPtr = true );
	//! Destructor.
										~CP_MacOSPtr();
	//@}

	//! @name Copying
	//@{
				void					CopyTo(
												void *inBuffer,
												UInt32 inNumBytes ) const;
				void					CopyFrom(
												void *ioBuffer,
												UInt32 inNumBytes );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the OS memory block.
										*	
										*	@exception	none
										*	
										*	@return		Ptr		The OS memory block.
										*/
				Ptr						GetOSPtr()									{ return( fBlock ); }
				char					*GetPtr();
	//@}

	//! @name Length
	//@{
				UInt32					Length() const;
				void					SetLength( UInt32 inLength );
	//@}

	//! @name Locking
	//@{
				void					Lock();
				void					Unlock();

										/*!
										*	Returns true if the memory block is locked.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if the memory block is locked.
										*/
				bool					IsLocked()									{ return( fLocked ); }
	//@}

	//! @name Operators
	//@{
			CP_MacOSPtr&				operator = (CP_MacOSPtr& inPtrBlock );

										/*!
										*	Returns the MacOS Handle.
										*	
										*	@exception	none
										*	
										*	@return		OSHandle
										*/
										operator Ptr()								{ return( fBlock ); }
	//@}
				
	//! @name Testers
	//@{
										/*!
										*	Returns true if the memory block is not NULL.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if the memory block is not NULL.
										*/
				bool					IsValid()									{ return( fBlock != NULL ); }

										/*!
										*	Returns true if this object owns the block of memory, and is therefore
										*	responsible for its deletion. Otherwise the memory block held by this object
										*	will not be deleted by this class.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if this object owns the memory handle.
										*/
				bool					OwnsPtr()									{ return( fOwnsPtr ); }
	//@}

	//! @name Wrapping
	//@{
				void					Wrap(
											Ptr inExistingPtr,
											bool inOwnsPtr = true );

										/*!
										*	Notifies this class that it no longer owns the memory, and will not
										*	delete it. You are now responsible for deleting the memory to avoid a
										*	memory leak.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
				void					Detach()									{ fOwnsPtr = false; }
	//@}

private:
	//! @name Allocation
	//@{
				void					Allocate( UInt32 inSize );
				void					Deallocate();
	//@}

	// class data
protected:
	Ptr									fBlock;					/*! Platforms memory object.*/
	bool								fOwnsPtr;				/*! Flag to indicate who is responsible for deleting the platforms memory.*/
	bool								fLocked;				/*! Flag to indicate if the block is locked.*/
								
	UInt32								fActualBlockLength;		/*! Length of the memory block.*/
};

#endif	// TARGET_OS_MAC || CP_Optional_QuickTime_Windows

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_MacOSPtr



