//=========================================================================
//
//	FILE:				CP_NetProtoBase.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base class for implementing internet protocol classes (FTP, HTTP, SMTP, etc).
//
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 22 $
//						Latest Checkout:  $Date: 2004-12-28 19:35:48 -0800 (Tue, 28 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_NetProtoBase
#define _H_CP_NetProtoBase
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_InternetResponse.h"
#include "CP_Socket.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_EXPORT CP_InternetProtocolProgress;

#if TARGET_CC_METROWERKS
#pragma mark === CP_NetProtoBase ===
#endif

/*!
*	@class		CP_NetProtoBase
*	@brief		Base class for implementing various internet protocols, such as SMTP, FTP, POP3, etc.
*/
class CP_EXPORT CP_NetProtoBase : public CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_NetProtoBase( CP_Socket *inSocketConnection );
	//! Destructor.
										~CP_NetProtoBase();
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the last server response.
										*	
										*	@exception	none
										*
										*	@return		CP_InternetResponse *	The internet response for this protocol.
										*/
	virtual		CP_InternetResponse		*GetServerResponse()						{ return( fLastServerResponse ); }
	//@}

	//! @name Progress
	//@{
				void					BroadcastProgress( CP_InternetProtocolProgress& inProtocolProgress );
	//@}

protected:
	//! @name Misc
	//@{
				SInt32					ReadLine(
												CP_Socket *inSocket,
												CP_String& inOutResult );
	//@}

	//! @name Getters
	//@{
				bool					IsCommaDelimitedListOfSixOctets( char *p );
	
				void					SendCommand( CP_String& inCommand );
				void					SendCommandWithReply( CP_String& inCommand );

				void					SendData( CP_String& inData );
				void					SendData(
												char *inData,
												UInt32 inDataLength );
	//@}
				
private:

	// class data
protected:
	CP_Socket							*fSocketConnection;		/*! The CP_Socket object that communicates between CPLAT and the server. */
	
	CP_InternetResponse					*fLastServerResponse;	/*! This object contains the last response for the server, each
																	different protocol class will implement its own response class
																	to handle server responses. */

	CP_DataValueSignal					fProgressSignal;		/*! Signal used to allow user to monitor progress. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark === CP_NetProtoBase ===
#endif

/*!
*	@class		CP_InternetProtocolProgress
*	@brief		Helper class to send progress information in a CP_Message about the status of
*				operations in internet protocol classes.
*/
class CP_EXPORT CP_InternetProtocolProgress
{
public: 
	enum CP_E_ProtocolProgress
	{
		CP_NetProto_OpeningConnection,
		CP_NetProto_ConnectionEstablished,
		CP_NetProto_ConnectionDisconnected,
		CP_NetProto_BeginAuthentication,
		CP_NetProto_AuthenticationComplete,
		CP_NetProto_OpeningDataConnection,
		CP_NetProto_DataConnectionEstablished,
		CP_NetProto_ClosingDataConnection,
		CP_NetProto_BeginDataTransfer,
		CP_NetProto_TransferedMoreBytes,
		CP_NetProto_EndDataTransfer,
		CP_NetProto_TransferComplete,
		CP_NetProto_TransferItemFail,
		CP_NetProto_RetrieveItemSuccess,
		CP_NetProto_RetrieveItemFail
	};

	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_InternetProtocolProgress();
										CP_InternetProtocolProgress(
												CP_NetProtoBase *inNetProtoBase,
												CP_E_ProtocolProgress inProgressMsg );
										CP_InternetProtocolProgress( const CP_InternetProtocolProgress& inAddress );
	//! Destructor.
	virtual								~CP_InternetProtocolProgress() throw();
	//@}

	//! @name operators
	//@{
			CP_InternetProtocolProgress& operator = ( const CP_InternetProtocolProgress& inAddress );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the internet protocol that this class is used with.
										*	
										*	@exception	none
										*
										*	@return		CP_NetProtoBase		The class that is used with the one.
										*/
			CP_NetProtoBase				*GetProtocol()									{ return( fProtocol ); }

										/*!
										*	Returns the progress message we are sending.
										*	
										*	@exception	none
										*
										*	@return		UInt32		The progress message being sent.
										*/
			UInt32						GetProgressMsg()								{ return( fProgressMessage ); }

										/*!
										*	Returns the total item count.
										*	
										*	@exception	none
										*
										*	@return		UInt32		The total item count.
										*/
			UInt32						GetTotalItem()									{ return( fTotalItems ); }

										/*!
										*	Returns the number of completed items.
										*	
										*	@exception	none
										*
										*	@return		UInt32		The number of completed items.
										*/
			UInt32						GetCompletedItems()								{ return( fCompletedItems ); }

										/*!
										*	Returns the total size in bytes of the item we are show progress for.
										*	
										*	@exception	none
										*
										*	@return		UInt32		The total size in bytes of the item.
										*/
			UInt32						GetTotalBytes()									{ return( fTotalBytes ); }

										/*!
										*	Returns the number of completed bytes.
										*	
										*	@exception	none
										*
										*	@return		UInt32		The number of completed bytes.
										*/
			UInt32						GetCompletedBytes()								{ return( fCompletedBytes ); }
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the protocol object we are monitoring progress for.
										*
										*	@param		inProtoBase		The CP_NetProtoBase we are monitoring.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetProtocol( CP_NetProtoBase *inProtoBase )		{ fProtocol = inProtoBase; }

										/*!
										*	Sets the message we will broadcast.
										*
										*	@param		inProgressMsg		The message to be broadcast.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetProgressMessage( UInt32 inProgressMsg )		{ fProgressMessage = inProgressMsg; }

										/*!
										*	Sets the total item count.
										*
										*	@param		inTotalItems		The total item count.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetTotalItem( UInt32 inTotalItems )				{ fTotalItems = inTotalItems; }

										/*!
										*	Sets the number of completed items so far.
										*
										*	@param		inCompletedItems	The number of completed items.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetCompletedItems( UInt32 inCompletedItems )	{ fCompletedItems = inCompletedItems; }

										/*!
										*	Sets the total number of bytes.
										*
										*	@param		inTotalBytes		The total number of bytes.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetTotalBytes( UInt32 inTotalBytes )			{ fTotalBytes = inTotalBytes; }

										/*!
										*	Sets the number of completed bytes.
										*
										*	@param		inCompletedBytes	The number of completed bytes.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetCompletedBytes( UInt32 inCompletedBytes)		{ fCompletedBytes = inCompletedBytes; }
	//@}

	// Data members
protected:
	CP_NetProtoBase						*fProtocol;				/*! Protocol class we are showing progress for. */

	UInt32								fProgressMessage;		/*! Message that will be sent. */
	UInt32								fTotalItems;			/*! Total number of items to show progress for. */
	UInt32								fCompletedItems;		/*! Number of completed items so far. */
	UInt32								fTotalBytes;			/*! Total number of bytes to show progress for. */
	UInt32								fCompletedBytes;		/*! Completed number of bytes so far. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_NetProtoBase


