//=========================================================================
//
//	FILE:				CP_SignalSlotHolder.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Classes to hold and map a signal or slot with a CP_String
//						identifier.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 162 $
//						Latest Checkout:  $Date: 2005-04-28 15:05:07 -0700 (Thu, 28 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_SignalSlotHolder
#define _H_CP_SignalSlotHolder
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueHolder.h"
#include "CP_SignalSlot.h"
#include "CP_SignalSlot_Defines.h"
#include "CP_String.h"
	
	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

/*!	\fn typedef CP_DataValueSignal
 *	Signal typedef used to pass a data value around. 
 */
typedef CP_SigSlot::CP_Signal2<CP_DataValueHolder&, OSStatus&>								CP_DataValueSignal;
/*!	\fn typedef CP_DataValueSlot
 *	Slot typedef used to receive a data value from a signal. 
 */
typedef CP_SigSlot::CP_Slot2<CP_DataValueHolder&, OSStatus&>								CP_DataValueSlot;

#if TARGET_CC_METROWERKS
#pragma mark === CP_SignalHolder ==
#endif

/*!
*	@class	CP_SignalHolder
*	@brief	Class to hold a CP_Signal1 object and CP_String used in storing
*			signals in lists that can be retreived by a signal name.
*	
*	This class stores an association of a single CP_Signal1 and a string containing
*	the name of the signal. Used in CP_Object to find and register signal handlers
*	by name.
*/
class CP_SignalHolder
{
public:
										/*!
										*	Constructor.
										*/
										CP_SignalHolder()									{}

										/*!
										*	Constructor.
										*	
										*	@param	inSignalName		String containing the name of the signal stored in inSignal.
										*	@param	inSignal			CP_Signal1 object. This is a CP_Signal that takes a single CP_DataValue parameter.
										*/
										CP_SignalHolder(
												const CP_String& inSignalName,
												CP_DataValueSignal *inSignal )
													: fSignalName(inSignalName), fSignal(inSignal)	{}

										/*!
										*	Copy constructor. Assigns the data fields from the passed in CP_SignalHolder.
										*	
										*	@param	inSignalHolder		CP_SignalHolder to copy the data fields from.
										*/
										CP_SignalHolder( const CP_SignalHolder& inSignalHolder )
											{
												fSignalName = inSignalHolder.fSignalName;
												fSignal = inSignalHolder.fSignal;
											};

										/*!
										*	Destructor.
										*/
										~CP_SignalHolder()									{};

									
										/*!
										*	Assignment operator.
										*	
										*	@param	inSignalHolder		CP_SignalHolder to copy the data fields from.
										*
										*	@return	CP_SignalHolder&	Returns a reference to ourselves.
										*/
	CP_SignalHolder&					operator = ( const CP_SignalHolder& inSignalHolder )
											{
												fSignalName = inSignalHolder.fSignalName;
												fSignal = inSignalHolder.fSignal;
											
												return( *this );
											}

										/*!
										*	Equality operator.
										*	
										*	@param	inSignalHolder		CP_SignalHolder to test for equality.
										*
										*	@return	bool				Returns true if this object is the same as the passed in CP_SignalHolder.
										*/
	bool								operator == ( const CP_SignalHolder& inSignalHolder )
											{
												bool			result = false;
											
												if( (fSignalName == inSignalHolder.fSignalName) &&
													(fSignal == inSignalHolder.fSignal) )
													result = true;
											
												return( result );
											}
		
										/*!
										*	Inequality operator.
										*	
										*	@param	inSignalHolder		CP_SignalHolder to test for inequality.
										*
										*	@return	bool				Returns true if this object are not the same as the passed in CP_SignalHolder.
										*/
	bool								operator != ( const CP_SignalHolder& inSignalHolder )
											{
												bool			result = false;
											
												if( (fSignalName != inSignalHolder.fSignalName) &&
													(fSignal != inSignalHolder.fSignal) )
													result = true;
											
												return( result );
											}

										/*!
										*	Returns the name of the signal.
										*
										*	@return	CP_String const		Returns the name of the signal as a CP_String.
										*/
			CP_String					GetSignalName() const								{ return( fSignalName ); }

										/*!
										*	Returns the CP_Signal1 signal object
										*
										*	@return	CP_DataValueSignal *	Returns a pointer to the CP_Signal1 object.
										*/
			CP_DataValueSignal			*GetSignal()										{ return( fSignal ); }

protected:
		// data members
	CP_String							fSignalName;		/*! CP_String containing the name of the signal. */

	CP_DataValueSignal					*fSignal;			/*! CP_DataValueSignal signal object. */
};

#if TARGET_CC_METROWERKS
#pragma mark === CP_SlotHolder ==
#endif

/*!
*	@class	CP_SlotHolder
*	@brief	Class to hold a CP_Slot1 object and CP_String used in storing
*			slots in lists that can be retreived by a slot name.
*	
*	This class stores an association of a single CP_Slot1 and a string containing
*	the name of the slot. Used in CP_Object to find and register slots by name.
*/
class CP_SlotHolder
{
public:
										/*!
										*	Constructor.
										*/
										CP_SlotHolder()										{}

										/*!
										*	Constructor.
										*	
										*	@param	inSlotName			String containing the name of the slot stored in inSlot.
										*	@param	inSignal			CP_Signal1 object. This is a CP_Slot that takes a single CP_DataValue parameter.
										*/
										CP_SlotHolder(
												const CP_String& inSlotName,
												CP_DataValueSlot inSlot )
													: fSlotName(inSlotName), fSlot(inSlot)	{}

										/*!
										*	Copy constructor. Assigns the data fields from the passed in CP_SlotHolder.
										*	
										*	@param	inSlotHolder		CP_SlotHolder to copy the data fields from.
										*/
										CP_SlotHolder( const CP_SlotHolder& inSlotHolder )
											{
												fSlotName = inSlotHolder.fSlotName;
												fSlot = inSlotHolder.fSlot;
											};

										/*!
										*	Destructor.
										*/
										~CP_SlotHolder()									{};
									
										/*!
										*	Assignment operator.
										*	
										*	@param	inSlotHolder		CP_SlotHolder to copy the data fields from.
										*
										*	@return	CP_SlotHolder&		Returns a reference to ourselves.
										*/
	CP_SlotHolder&						operator = ( const CP_SlotHolder& inSlotHolder )
											{
												fSlotName = inSlotHolder.fSlotName;
												fSlot = inSlotHolder.fSlot;
											
												return( *this );
											}

										/*!
										*	Equality operator.
										*	
										*	@param	inSlotHolder		CP_SlotHolder to test for equality.
										*
										*	@return	bool				Returns true if this object is the same as the passed in CP_SignalHolder.
										*/
	bool								operator == ( const CP_SlotHolder& inSlotHolder )
											{
												bool			result = false;
											
												if( fSlotName == inSlotHolder.fSlotName )
													result = true;
											
												return( result );
											}

										/*!
										*	Inequality operator.
										*	
										*	@param	inSlotHolder		CP_SlotHolder to test for inequality.
										*
										*	@return	bool				Returns true if this object are not the same as the passed in CP_SignalHolder.
										*/
	bool								operator != ( const CP_SlotHolder& inSlotHolder )
											{
												bool			result = false;
											
												if( fSlotName != inSlotHolder.fSlotName )
													result = true;
											
												return( result );
											}

										/*!
										*	Returns the name of the slot.
										*
										*	@return	CP_String const		Returns the name of the slot as a CP_String.
										*/
			CP_String					GetSlotName() const									{ return( fSlotName ); }

										/*!
										*	Returns the CP_DataValueSlot slot object
										*
										*	@return	CP_DataValueSlot		Returns a CP_DataValueSlot object.
										*/
			CP_DataValueSlot			GetSlot()											{ return( fSlot ); }

protected:
		// data members
	CP_String							fSlotName;		/*! CP_String containing the name of the slot. */
	
	CP_DataValueSlot					fSlot;			/*! The CP_DataValueSlot slot object. */
};

/*!	\fn typedef CP_SignalsArray
 *	A std::vector object that holds a list of CP_SignalHolders. 
 */
typedef std::vector<CP_SignalHolder, std::allocator<CP_SignalHolder> >								CP_Signals_Array;
typedef std::vector<CP_SignalHolder, std::allocator<CP_SignalHolder> >::iterator					CP_Signals_Iterator;

/*!	\fn typedef CP_SlotsArray
 *	A std::vector object that holds a list of CP_SlotHolders. 
 */
typedef std::vector<CP_SlotHolder, std::allocator<CP_SlotHolder> >									CP_Slots_Array;
typedef std::vector<CP_SlotHolder, std::allocator<CP_SlotHolder> >::iterator						CP_Slots_Iterator;

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_SignalSlotHolder

