//=========================================================================
//
//	FILE:				CP_InStream.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Classes that provide streaming support for reading data
//						from streams. There are classes that will stream the
//						data to a memory block or file.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_InStream
#define _H_CP_InStream
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_RGBColor.h"
#include "CP_StreamDefines.h"
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

class CP_HandleBlock;
class CP_PtrBlock;

/*!
*	@class		CP_Instream
*	@brief		Abstract streaming class for reading data from a stream.
*
*				This class can be used to read data that is streamed out by the CPLAT
*				CP_OutStream class or from other data sources.
*				If the data comes from a CP_OutStream, the stream data will have a
*				header written before any data. The format of the header data is:
*
*	@code
*					char		cplatEdian = kCP_StreamData_BigEdian;	// or kCP_StreamData_LittleEdian
*					SInt32		cplatSignature = kCP_StreamHeaderTag;	// 'CPLA'
*					SInt16		streamVersion = 1;						// current stream version is 1
*					char		hasTags = false;						// no debug tags
*	@endcode
*
*				In addition to this class, there are CP_InStream and CP_OutStream classes
*				that can write the data to a CP_HandleBlock object, or a CP_File.
*/
class CP_EXPORT CP_InStream
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_InStream(
												bool inHasHeader = true,
												char inDesiredEdian = kCP_StreamData_PlatformEdian );
										CP_InStream( const CP_InStream& inStream );
	//! Destructor.
	virtual								~CP_InStream() throw();
	//@}

	//! @name Reading
	//@{
	virtual UInt32 						ReadBytes(
												void *inDestBuffer,
												UInt32 inNumberBytes,
												bool inAllowPartialData=false);
	//@}

	//! @name ByteSwapping
	//@{
			bool 						NeedsByteSwap() const						{ return( fNeedsByteSwap ); }
			void						SetNeedsByteSwap( bool doSwap )				{ fNeedsByteSwap = doSwap; }
	//@}

	//! @name Getters
	//@{
	virtual UInt32						GetLength() const = 0;
			UInt16						GetStreamVersion() const					{ return( fCPLATStreamVersion ); }
			bool						HasHeader() const							{ return fHasHeader; }
	//@}
			
	//! @name Positioning
	//@{
	virtual UInt32						GetPosition() const = 0;
	virtual void						SetPosition( UInt32 inPosition ) = 0;
	virtual	bool 						IsEOF() const;
	//@}

	//! @name Operators
	//@{
			CP_InStream&				operator = ( const CP_InStream& inStream );
	//@}

	//! @name Tags
	//@{
			bool 						HasTaggedData() const;
			SInt32 						GetTag();
	//@}

protected:
	//! @name Reading
	//@{
	virtual UInt32 						OnReadBytes(
											void *inDestBuffer,
											UInt32 inNumberBytes ) = 0;

	virtual void 						ReadHeader();
	//@}

protected:
	bool								fHasReadHeader;			/*! Flag to indicate if we have read the header. */
	bool								fNeedsByteSwap;			/*! Flag to indicate if we need to byte swap the data. */

	UInt16								fCPLATStreamVersion;	/*! CPLAT stream version used to write this stream. */
	bool								fHasTaggedData;			/*! Flag to indicate if the data has its type written out also. */

	bool								fHasHeader;				/*! Flag to indicate if we are want to read data not created by
																	this framework and therefore won't have a correct header. */

	char								fDesiredEdian;			/*! The edian value you want the data in. See ctor comments in source for full details. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CP_InStream& operator >> ( CP_InStream& stream, bool& inBool );		
CP_InStream& operator >> ( CP_InStream& stream, char& inchar );				
CP_InStream& operator >> ( CP_InStream& stream, unsigned char& inUchar );
CP_InStream& operator >> ( CP_InStream& stream, char *inchar );
CP_InStream& operator >> ( CP_InStream& stream, unsigned char *inchar );
CP_InStream& operator >> ( CP_InStream& stream, SInt16& inSInt16 );
CP_InStream& operator >> ( CP_InStream& stream, UInt16& inUInt16 );
CP_InStream& operator >> ( CP_InStream& stream, SInt32& inSInt32 );
CP_InStream& operator >> ( CP_InStream& stream, UInt32& inUInt32 );
CP_InStream& operator >> ( CP_InStream& stream, float& inFloat );
CP_InStream& operator >> ( CP_InStream& stream, double& inDouble );
CP_InStream& operator >> ( CP_InStream& stream, CP_String& inString );
CP_InStream& operator >> ( CP_InStream& stream, CP_Point& inPoint );
CP_InStream& operator >> ( CP_InStream& stream, CP_Size& inSize );
CP_InStream& operator >> ( CP_InStream& stream, CP_Rect& inRect);

CP_InStream& operator >> ( CP_InStream& stream, CP_RGBColor& inRGBColor );
CP_InStream& operator >> ( CP_InStream& stream, CP_HandleBlock& inHandle );
CP_InStream& operator >> ( CP_InStream& stream, CP_PtrBlock& inPtr );
CP_InStream& operator >> ( CP_InStream& inStream, CP_FileSystemSpec& inFileSpec );

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_InStream


