//=========================================================================
//
//	FILE:				CP_OpenGL_Utilities.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A set of utilities and classes for working with OpenGL
//						in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_OpenGL_Utilities
#define _H_CP_OpenGL_Utilities
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework

#if TARGET_OS_MAC
	#if TARGET_API_MAC_CARBON && __CF_USE_FRAMEWORK_INCLUDES__
		#include <agl/agl.h>
	#else
		#include "agl.h"
	#endif
#endif

#if TARGET_OS_WIN32
	#include <GL/gl.h>
	#include <GL/glu.h>
#endif

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC

/*!
*	@class		CP_AglContext
*	@brief		A class for management of an OpenGL AglContext.
*
*				This class is used in the OpenGLView to provide an AglContext for the
*				OpenGL view to draw into.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_AglContext
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AglContext();
										CP_AglContext( CP_AglContext& inContext );
			explicit					CP_AglContext( AGLContext inContext );
	//! Destructor.
										~CP_AglContext() throw();
	//@}
	
	//! @name Context Management
	//@{
										/*!
										*	Returns the AglContext.
										*
										*	@exception	none
										*	
										*	@return		AglContext		The AglContext we are managing.
										*/
			AGLContext					Get() const throw()							{ return( fAGLContext ); }
			AGLContext					Release() throw();
			void						Reset( AGLContext inContext = NULL ) throw();
	//@}

	//! @name operators
	//@{
			CP_AglContext&				operator = ( CP_AglContext& inContext ) throw();

										/*!
										*	Returns the AglContext.
										*
										*	@exception	none
										*	
										*	@return		AglContext		The AglContext we are managing.
										*/
										operator AGLContext () const throw()		{ return( fAGLContext ); }
	//@}

	// class data
protected:
	AGLContext							fAGLContext;		/* ! OpenGL AGLContext we are managing */
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

/*!
*	@class		CP_StSetAglContext
*	@brief		A stack based class for setting and restoring an AglContext.
*
*				This class is designed to be used as a stack based class to set and restore
*				an OpenGL AglContext for drawing into.
*
*	@code
*				void MyOpenGLDrawing()
*				{
*					CP_StSetAglContext			savedAGLContext( fAGLContext );
*				
*						Draw the OpenGL content here
*									
*					glFlush();
*				}
*	@endcode
*
*				In the above example, the current context will be set to fAGLContext, and when
*				the routine ends, the AglContext will be restored to what ever it was when the
*				above function was entered.
*
*	@ingroup	UI
*/
class CP_StSetAglContext : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_StSetAglContext( AGLContext inContext );
	//! Destructor.
										~CP_StSetAglContext() throw();
	//@}

	// class data
private:
	AGLContext							fSavedAGLContext;		/* ! Existing AGLContext to restore on exit. */
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#endif

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_OpenGL_Utilities
