//=========================================================================
//
//	FILE:				CP_WindowListHeader.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms window list header control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 203 $
//						Latest Checkout:  $Date: 2005-06-04 13:40:08 -0700 (Sat, 04 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_WindowListHeader
#define _H_CP_WindowListHeader
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"

CPLAT_Begin_Namespace_CPLAT

// ===================================================================================
//	CP_E_WindoHeaderStyle - defines the style used to display the items.
// ===================================================================================
enum kCP_E_WindowHeaderStyle
{
	kCP_E_WindowHeaderStyle_Default = 0,				// Flat style
	kCP_E_WindowHeaderStyle_Buttons = 0x0002,			// HDS_BUTTONS
	kCP_E_WindowHeaderStyle_HotTrack = 0x0004			// HDS_HOTTRACK
};

// ===================================================================================
//	CP_S_WindoListItemInfo - structure passed into our window list header routines with
//		information about a column in the list.
// ===================================================================================
struct CP_S_WindoListItemInfo
{
	SInt16					fImageResID;
	
	SInt16					fColumnWidth;
    SInt16					fColumnMinSize;			// minimum size column can go
	
	UInt32					fCommandID;				// command sent when clicked on
	
	SInt32					fUserData;
	SInt32					fTextAlign;
	CP_String				fText;
	
	bool					fSortDirection;
	
							CP_S_WindoListItemInfo();
							CP_S_WindoListItemInfo(
									const CP_String& inText,
									UInt32 inCommandID,
									SInt32 inSInt32 = CP_E_TextAlign_HorLeft,
									const SInt16 inImageResID = kCP_NoImageIndex,
									const SInt16 inColumnWidth = 10,
									SInt32 inUserData = 0,
									SInt16 inColMinSize = 1 );
							CP_S_WindoListItemInfo( const CP_S_WindoListItemInfo& inTabProperty );

	CP_S_WindoListItemInfo&	operator =( const CP_S_WindoListItemInfo& inTabProperty );
};

#define kCP_WindowListHeaderEdgeOffset			3

	// typedefs
#if TARGET_OS_LINUX
	typedef std::vector<CP_S_WindoListItemInfo, std::allocator<CP_S_WindoListItemInfo> >			CP_WindowListItem_Array;
	typedef std::vector<CP_S_WindoListItemInfo, std::allocator<CP_S_WindoListItemInfo> >::iterator	CP_WindowListItem_Iterator;
#else
	typedef std::vector<CP_S_WindoListItemInfo >													CP_WindowListItem_Array;
	typedef std::vector<CP_S_WindoListItemInfo >::iterator											CP_WindowListItem_Iterator;
#endif

/*!
*	@class		CP_WindowListHeader
*	@brief		Implementation of a platforms window list header control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_WindowListHeader : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_WindowListHeader();
										CP_WindowListHeader(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												kCP_E_WindowHeaderStyle inStyle = kCP_E_WindowHeaderStyle_Default );
	//! Destructor.
	virtual								~CP_WindowListHeader() throw();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns the current sort direction.
										*	
										*	@exception	none
										*	
										*	@return		bool		True if the sort direction is in ascending order.
										*/
			bool						GetSortDirection()												{ return( fSortDirection ); }
	//@}

	//! @name Items
	//@{
			void						AddItem( CP_S_WindoListItemInfo& inItemInfo );
			void						InsertItem(
												CP_S_WindoListItemInfo& inItemInfo,
												UInt32 inIndex );
			void						DeleteItem( UInt32 inIndex );

			void						SetItem(
												const CP_S_WindoListItemInfo& inItemInfo,
												UInt32 inIndex );
			CP_S_WindoListItemInfo		GetItem( const UInt32 inColumn );

			UInt32						GetItemCount();
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
	//! @name Setters
	//@{
										/*!
										*	Sets the current sort direction.
										*
										*	@paraam		inSortDirection		The current sorting direction.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetSortDirection( bool inSortDirection )							{ inSortDirection = fSortDirection; }
	//@}

protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );

			void						DrawOneItem(
												CP_Canvas& inCanvas,
												CP_Rect& inBounds,
												const CP_S_WindoListItemInfo& inColumnInfo,
												bool inIsSelected,
												bool inIsPressed );
			void						CalcRects(
												CP_Canvas& inCanvas,
												CP_Rect& inBounds,
												const CP_S_WindoListItemInfo& inColumnInfo,
												CP_Rect& inOutTextBounds,
												CP_Rect& inOutImageBounds );
			void						DrawColumnDivider(
												CP_Canvas& inCanvas,
												const CP_Rect& inBounds );
	//@}

	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

private:
	//! @name Slots
	//@{
			void						HandleControlHitEvent( CP_Event_Base& inEvent );
			void						HandleMouseUpEvent( CP_Event_Base& inEvent );
	//@}

	//! @name Events
	//@{
			void						HandleClickInColumn( const CP_Point& inLocalMouseClick );
	//@}

	//! @name Misc
	//@{
			SInt16						FindColumnFromPoint( const CP_Point& inLocalMouseClick );
	//@}

	// class data
protected:
	CP_WindowListItem_Array				*fItems;
	
	kCP_E_WindowHeaderStyle				fStyle;
		
	bool								fSortDirection;

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}

	//! @name Initialization
	//@{
			void						HandleValueChangedEvent( CP_Event_Base& inEvent );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_WindowListHeader
